@@ -95,6 +95,12 @@ public:
      */
     KNewFileMenu* newFileMenu() const;
 
+    /**
+     * Augments Qt's build-in QMainWindow context menu to add
+     * Dolphin-specific actions, such as "unlock panels".
+     */
+    QMenu *createPopupMenu() override;
+
     /**
      * Switch the window's view containers' locations to display the home path
      * for any which are currently displaying a location corresponding to or
@@ -171,6 +177,16 @@ public Q_SLOTS:
      */
     void openNewTab(const QUrl& url);
 
+    /**
+     * Opens a new tab  showing the URL \a url and activate it.
+     */
+    void openNewTabAndActivate(const QUrl &url);
+
+    /**
+     * Opens a new window showing the URL \a url.
+     */
+    void openNewWindow(const QUrl &url);
+
     /** @see GeneralSettings::splitViewChanged() */
     void slotSplitViewChanged();
 
@@ -566,6 +582,12 @@ private Q_SLOTS:
       * to go to.
       */
     void slotGoForward(QAction* action);
+
+    /**
+     * Is called when configuring the keyboard shortcuts
+     */
+    void slotKeyBindings();
+
 private:
     /**
      * Sets up the various menus and actions and connects them.
