@@ -46,7 +46,7 @@ KItemListController::KItemListController(KItemModelBase* model, KItemListView* v
     m_view(nullptr),
     m_selectionManager(new KItemListSelectionManager(this)),
     m_keyboardManager(new KItemListKeyboardSearchManager(this)),
-    m_pressedIndex(-1),
+    m_pressedIndex(std::nullopt),
     m_pressedMousePos(),
     m_autoActivationTimer(nullptr),
     m_swipeGesture(Qt::CustomGesture),
@@ -574,16 +574,16 @@ bool KItemListController::mouseMoveEvent(QGraphicsSceneMouseEvent* event, const
         return false;
     }
 
-    if (m_pressedIndex >= 0) {
+    if (m_pressedIndex.has_value() && !m_view->rubberBand()->isActive()) {
         // Check whether a dragging should be started
         if (event->buttons() & Qt::LeftButton) {
             const QPointF pos = transform.map(event->pos());
             if ((pos - m_pressedMousePos).manhattanLength() >= QApplication::startDragDistance()) {
-                if (!m_selectionManager->isSelected(m_pressedIndex)) {
+                if (!m_selectionManager->isSelected(m_pressedIndex.value())) {
                     // Always assure that the dragged item gets selected. Usually this is already
                     // done on the mouse-press event, but when using the selection-toggle on a
                     // selected item the dragged item is not selected yet.
-                    m_selectionManager->setSelected(m_pressedIndex, 1, KItemListSelectionManager::Toggle);
+                    m_selectionManager->setSelected(m_pressedIndex.value(), 1, KItemListSelectionManager::Toggle);
                 } else {
                     // A selected item has been clicked to drag all selected items
                     // -> the selection should not be cleared when the mouse button is released.
@@ -599,12 +599,12 @@ bool KItemListController::mouseMoveEvent(QGraphicsSceneMouseEvent* event, const
             QPointF endPos = transform.map(event->pos());
 
             // Update the current item.
-            const int newCurrent = m_view->itemAt(endPos);
-            if (newCurrent >= 0) {
+            const std::optional<int> newCurrent = m_view->itemAt(endPos);
+            if (newCurrent.has_value()) {
                 // It's expected that the new current index is also the new anchor (bug 163451).
                 m_selectionManager->endAnchoredSelection();
-                m_selectionManager->setCurrentItem(newCurrent);
-                m_selectionManager->beginAnchoredSelection(newCurrent);
+                m_selectionManager->setCurrentItem(newCurrent.value());
+                m_selectionManager->beginAnchoredSelection(newCurrent.value());
             }
 
             if (m_view->scrollOrientation() == Qt::Vertical) {
@@ -642,7 +642,7 @@ bool KItemListController::mouseReleaseEvent(QGraphicsSceneMouseEvent* event, con
         return false;
     }
 
-    Q_EMIT mouseButtonReleased(m_pressedIndex, event->buttons());
+    Q_EMIT mouseButtonReleased(m_pressedIndex.value_or(-1), event->buttons());
 
     return onRelease(transform.map(event->pos()), event->modifiers(), event->button(), false);
 }
@@ -650,21 +650,21 @@ bool KItemListController::mouseReleaseEvent(QGraphicsSceneMouseEvent* event, con
 bool KItemListController::mouseDoubleClickEvent(QGraphicsSceneMouseEvent* event, const QTransform& transform)
 {
     const QPointF pos = transform.map(event->pos());
-    const int index = m_view->itemAt(pos);
+    const std::optional<int> index = m_view->itemAt(pos);
 
     // Expand item if desired - See Bug 295573
     if (m_mouseDoubleClickAction != ActivateItemOnly) {
-        if (m_view && m_model && m_view->supportsItemExpanding() && m_model->isExpandable(index)) {
-            const bool expanded = m_model->isExpanded(index);
-            m_model->setExpanded(index, !expanded);
+        if (m_view && m_model && m_view->supportsItemExpanding() && m_model->isExpandable(index.value_or(-1))) {
+            const bool expanded = m_model->isExpanded(index.value());
+            m_model->setExpanded(index.value(), !expanded);
         }
     }
 
     if (event->button() & Qt::RightButton) {
         m_selectionManager->clearSelection();
-        if (index >= 0) {
-            m_selectionManager->setSelected(index);
-            Q_EMIT itemContextMenuRequested(index, event->screenPos());
+        if (index.has_value()) {
+            m_selectionManager->setSelected(index.value());
+            Q_EMIT itemContextMenuRequested(index.value(), event->screenPos());
         } else {
             const QRectF headerBounds = m_view->headerBoundaries();
             if (headerBounds.contains(event->pos())) {
@@ -678,9 +678,9 @@ bool KItemListController::mouseDoubleClickEvent(QGraphicsSceneMouseEvent* event,
 
     bool emitItemActivated = !(m_view->style()->styleHint(QStyle::SH_ItemView_ActivateItemOnSingleClick) || m_singleClickActivationEnforced) &&
                              (event->button() & Qt::LeftButton) &&
-                             index >= 0 && index < m_model->count();
+                             index.has_value() && index.value() < m_model->count();
     if (emitItemActivated) {
-        Q_EMIT itemActivated(index);
+        Q_EMIT itemActivated(index.value());
     }
     return false;
 }
@@ -805,7 +805,7 @@ bool KItemListController::dropEvent(QGraphicsSceneDragDropEvent* event, const QT
         Q_EMIT aboveItemDropEvent(dropAboveIndex, event);
     } else if (!event->mimeData()->hasFormat(m_model->blacklistItemDropEventMimeType())) {
         // Something has been dropped on an item or on an empty part of the view.
-        Q_EMIT itemDropEvent(m_view->itemAt(pos), event);
+        Q_EMIT itemDropEvent(m_view->itemAt(pos).value_or(-1), event);
     }
 
     QAccessibleEvent accessibilityEvent(view(), QAccessible::DragDropEnd);
@@ -828,27 +828,76 @@ bool KItemListController::hoverMoveEvent(QGraphicsSceneHoverEvent* event, const
         return false;
     }
 
-    KItemListWidget* oldHoveredWidget = hoveredWidget();
-    const QPointF pos = transform.map(event->pos());
-    KItemListWidget* newHoveredWidget = widgetForPos(pos);
+    // We identify the widget whose expansionArea had been hovered before this hoverMoveEvent() triggered.
+    // we can't use hoveredWidget() here (it handles the icon+text rect, not the expansion rect)
+    // like hoveredWidget(), we find the hovered widget for the expansion rect
+    const auto visibleItemListWidgets = m_view->visibleItemListWidgets();
+    const auto oldHoveredExpansionWidgetIterator = std::find_if(visibleItemListWidgets.begin(), visibleItemListWidgets.end(), [](auto &widget) {
+        return widget->expansionAreaHovered();
+    });
+    const auto oldHoveredExpansionWidget = oldHoveredExpansionWidgetIterator == visibleItemListWidgets.end() ?
+                                           std::nullopt : std::make_optional(*oldHoveredExpansionWidgetIterator);
 
-    if (oldHoveredWidget != newHoveredWidget) {
-        if (oldHoveredWidget) {
+    const auto unhoverOldHoveredWidget = [&]() {
+        if (auto oldHoveredWidget = hoveredWidget(); oldHoveredWidget) {
+            // handle the text+icon one
             oldHoveredWidget->setHovered(false);
             Q_EMIT itemUnhovered(oldHoveredWidget->index());
         }
+    };
 
-        if (newHoveredWidget) {
-            newHoveredWidget->setHovered(true);
-            const QPointF mappedPos = newHoveredWidget->mapFromItem(m_view, pos);
-            newHoveredWidget->setHoverPosition(mappedPos);
-            Q_EMIT itemHovered(newHoveredWidget->index());
+    const auto unhoverOldExpansionWidget = [&]() {
+        if (oldHoveredExpansionWidget) {
+            // then the expansion toggle
+            (*oldHoveredExpansionWidget)->setExpansionAreaHovered(false);
         }
-    } else if (oldHoveredWidget) {
-        const QPointF mappedPos = oldHoveredWidget->mapFromItem(m_view, pos);
-        oldHoveredWidget->setHoverPosition(mappedPos);
-    }
+    };
 
+    const QPointF pos = transform.map(event->pos());
+    if (KItemListWidget *newHoveredWidget = widgetForPos(pos); newHoveredWidget) {
+        // something got hovered, work out which part and set hover for the appropriate widget
+        const auto mappedPos = newHoveredWidget->mapFromItem(m_view, pos);
+        const bool isOnExpansionToggle = newHoveredWidget->expansionToggleRect().contains(mappedPos);
+
+        if (isOnExpansionToggle) {
+            // make sure we unhover the old one first if old!=new
+            if (oldHoveredExpansionWidget && *oldHoveredExpansionWidget != newHoveredWidget) {
+                (*oldHoveredExpansionWidget)->setExpansionAreaHovered(false);
+            }
+            // we also unhover any old icon+text hovers, in case the mouse movement from icon+text to expansion toggle is too fast (i.e. newHoveredWidget is never null between the transition)
+            unhoverOldHoveredWidget();
+
+
+            newHoveredWidget->setExpansionAreaHovered(true);
+        } else {
+            // make sure we unhover the old one first if old!=new
+            if (auto oldHoveredWidget = hoveredWidget(); oldHoveredWidget && oldHoveredWidget != newHoveredWidget) {
+                oldHoveredWidget->setHovered(false);
+                Q_EMIT itemUnhovered(oldHoveredWidget->index());
+            }
+            // we also unhover any old expansion toggle hovers, in case the mouse movement from expansion toggle to icon+text is too fast (i.e. newHoveredWidget is never null between the transition)
+            unhoverOldExpansionWidget();
+
+            const bool isOverIconAndText = newHoveredWidget->iconRect().contains(mappedPos) || newHoveredWidget->textRect().contains(mappedPos);
+            const bool hasMultipleSelection = m_selectionManager->selectedItems().count() > 1;
+
+            if (hasMultipleSelection && !isOverIconAndText) {
+                // In case we have multiple selections, clicking on any row will deselect the selection.
+                // So, as a visual cue for signalling that clicking anywhere won't select, but clear current highlights,
+                // we disable hover of the *row*(i.e. blank space to the right of the icon+text)
+
+                // (no-op in this branch for masked hover)
+            } else {
+                newHoveredWidget->setHovered(true);
+                newHoveredWidget->setHoverPosition(mappedPos);
+                Q_EMIT itemHovered(newHoveredWidget->index());
+            }
+        }
+    } else {
+        // unhover any currently hovered expansion and text+icon widgets
+        unhoverOldHoveredWidget();
+        unhoverOldExpansionWidget();
+    }
     return false;
 }
 
@@ -998,10 +1047,10 @@ void KItemListController::tapAndHoldTriggered(QGestureEvent* event, const QTrans
         m_pressedMousePos = transform.map(event->mapToGraphicsScene(tap->position()));
         m_pressedIndex = m_view->itemAt(m_pressedMousePos);
 
-        if (m_pressedIndex >= 0 && !m_selectionManager->isSelected(m_pressedIndex)) {
+        if (m_pressedIndex.has_value() && !m_selectionManager->isSelected(m_pressedIndex.value())) {
             m_selectionManager->clearSelection();
-            m_selectionManager->setSelected(m_pressedIndex);
-        } else if (m_pressedIndex == -1) {
+            m_selectionManager->setSelected(m_pressedIndex.value());
+        } else if (!m_pressedIndex.has_value()) {
             m_selectionManager->clearSelection();
             startRubberBand();
         }
@@ -1064,9 +1113,9 @@ void KItemListController::swipeTriggered(QGestureEvent* event, const QTransform&
         Q_EMIT scrollerStop();
 
         if (swipe->swipeAngle() <= 20 || swipe->swipeAngle() >= 340) {
-            Q_EMIT mouseButtonPressed(m_pressedIndex, Qt::BackButton);
+            Q_EMIT mouseButtonPressed(m_pressedIndex.value_or(-1), Qt::BackButton);
         } else if (swipe->swipeAngle() <= 200 && swipe->swipeAngle() >= 160) {
-            Q_EMIT mouseButtonPressed(m_pressedIndex, Qt::ForwardButton);
+            Q_EMIT mouseButtonPressed(m_pressedIndex.value_or(-1), Qt::ForwardButton);
         } else if (swipe->swipeAngle() <= 110 && swipe->swipeAngle() >= 60) {
             Q_EMIT swipeUp();
         }
@@ -1085,7 +1134,7 @@ void KItemListController::twoFingerTapTriggered(QGestureEvent* event, const QTra
     if (twoTap->state() == Qt::GestureStarted) {
         m_pressedMousePos = transform.map(twoTap->pos());
         m_pressedIndex = m_view->itemAt(m_pressedMousePos);
-        if (m_pressedIndex >= 0) {
+        if (m_pressedIndex.has_value()) {
             onPress(twoTap->screenPos().toPoint(), twoTap->pos().toPoint(), Qt::ControlModifier, Qt::LeftButton);
             onRelease(transform.map(twoTap->pos()), Qt::ControlModifier, Qt::LeftButton, false);
         }
@@ -1303,10 +1352,7 @@ KItemListWidget* KItemListController::widgetForPos(const QPointF& pos) const
     const auto widgets = m_view->visibleItemListWidgets();
     for (KItemListWidget* widget : widgets) {
         const QPointF mappedPos = widget->mapFromItem(m_view, pos);
-
-        const bool hovered = widget->contains(mappedPos) &&
-                             !widget->expansionToggleRect().contains(mappedPos);
-        if (hovered) {
+        if (widget->contains(mappedPos) || widget->selectionRect().contains(mappedPos)) {
             return widget;
         }
     }
@@ -1419,7 +1465,7 @@ void KItemListController::updateExtendedSelectionRegion()
 
 bool KItemListController::onPress(const QPoint& screenPos, const QPointF& pos, const Qt::KeyboardModifiers modifiers, const Qt::MouseButtons buttons)
 {
-    Q_EMIT mouseButtonPressed(m_pressedIndex, buttons);
+    Q_EMIT mouseButtonPressed(m_pressedIndex.value_or(-1), buttons);
 
     if (buttons & (Qt::BackButton | Qt::ForwardButton)) {
         // Do not select items when clicking the back/forward buttons, see
@@ -1427,26 +1473,27 @@ bool KItemListController::onPress(const QPoint& screenPos, const QPointF& pos, c
         return true;
     }
 
-    if (m_view->isAboveExpansionToggle(m_pressedIndex, m_pressedMousePos)) {
+    if (m_view->isAboveExpansionToggle(m_pressedIndex.value_or(-1), m_pressedMousePos)) {
         m_selectionManager->endAnchoredSelection();
-        m_selectionManager->setCurrentItem(m_pressedIndex);
-        m_selectionManager->beginAnchoredSelection(m_pressedIndex);
+        m_selectionManager->setCurrentItem(m_pressedIndex.value());
+        m_selectionManager->beginAnchoredSelection(m_pressedIndex.value());
         return true;
     }
 
-    m_selectionTogglePressed = m_view->isAboveSelectionToggle(m_pressedIndex, m_pressedMousePos);
+    m_selectionTogglePressed = m_view->isAboveSelectionToggle(m_pressedIndex.value_or(-1), m_pressedMousePos);
     if (m_selectionTogglePressed) {
-        m_selectionManager->setSelected(m_pressedIndex, 1, KItemListSelectionManager::Toggle);
+        m_selectionManager->setSelected(m_pressedIndex.value(), 1, KItemListSelectionManager::Toggle);
         // The previous anchored selection has been finished already in
         // KItemListSelectionManager::setSelected(). We can safely change
         // the current item and start a new anchored selection now.
-        m_selectionManager->setCurrentItem(m_pressedIndex);
-        m_selectionManager->beginAnchoredSelection(m_pressedIndex);
+        m_selectionManager->setCurrentItem(m_pressedIndex.value());
+        m_selectionManager->beginAnchoredSelection(m_pressedIndex.value());
         return true;
     }
 
     const bool shiftPressed = modifiers & Qt::ShiftModifier;
     const bool controlPressed = modifiers & Qt::ControlModifier;
+    const bool rightClick = buttons & Qt::RightButton;
 
     // The previous selection is cleared if either
     // 1. The selection mode is SingleSelection, or
@@ -1456,11 +1503,41 @@ bool KItemListController::onPress(const QPoint& screenPos, const QPointF& pos, c
     //       - start dragging multiple items, or
     //       - open the context menu and perform an action for all selected items.
     const bool shiftOrControlPressed = shiftPressed || controlPressed;
-    const bool pressedItemAlreadySelected = m_pressedIndex >= 0 && m_selectionManager->isSelected(m_pressedIndex);
+    const bool pressedItemAlreadySelected = m_pressedIndex.has_value() && m_selectionManager->isSelected(m_pressedIndex.value());
     const bool clearSelection = m_selectionBehavior == SingleSelection ||
                                 (!shiftOrControlPressed && !pressedItemAlreadySelected);
+
+
+    // When this method returns false, a rubberBand selection is created using KItemListController::startRubberBand via the caller.
     if (clearSelection) {
+        const int selectedItemsCount = m_selectionManager->selectedItems().count();
         m_selectionManager->clearSelection();
+        // clear and bail when we got an existing multi-selection
+        if (selectedItemsCount  > 1 && m_pressedIndex.has_value()) {
+            const auto row = m_view->m_visibleItems.value(m_pressedIndex.value());
+            const auto mappedPos = row->mapFromItem(m_view, pos);
+            if (pressedItemAlreadySelected || row->iconRect().contains(mappedPos) || row->textRect().contains(mappedPos)) {
+                // we are indeed inside the text/icon rect, keep m_pressedIndex what it is
+                // and short-circuit for single-click activation (it will then propagate to onRelease and activate the item)
+                // or we just keep going for double-click activation
+                if (m_view->style()->styleHint(QStyle::SH_ItemView_ActivateItemOnSingleClick) || m_singleClickActivationEnforced) {
+                    if (!pressedItemAlreadySelected) {
+                        // An unselected item was clicked directly while deselecting multiple other items so we select it.
+                        m_selectionManager->setSelected(m_pressedIndex.value(), 1, KItemListSelectionManager::Toggle);
+                        m_selectionManager->setCurrentItem(m_pressedIndex.value());
+                        m_selectionManager->beginAnchoredSelection(m_pressedIndex.value());
+                    }
+                    return true; // event handled, don't create rubber band
+                }
+            } else {
+                // we're not inside the text/icon rect, as we've already cleared the selection
+                // we can just stop here and make sure handlers down the line (i.e. onRelease) don't activate
+                m_pressedIndex.reset();
+                // we don't stop event propagation and proceed to create a rubber band and let onRelease
+                // decide (based on m_pressedIndex) whether we're in a drag (drag => new rubber band, click => don't select the item)
+                return false;
+            }
+        }
     } else if (pressedItemAlreadySelected && !shiftOrControlPressed && (buttons & Qt::LeftButton)) {
         // The user might want to start dragging multiple items, but if he clicks the item
         // in order to trigger it instead, the other selected items must be deselected.
@@ -1469,8 +1546,8 @@ bool KItemListController::onPress(const QPoint& screenPos, const QPointF& pos, c
         //    clear the selection in mouseReleaseEvent(), unless the items are dragged.
         m_clearSelectionIfItemsAreNotDragged = true;
 
-        if (m_selectionManager->selectedItems().count() == 1 && m_view->isAboveText(m_pressedIndex, m_pressedMousePos)) {
-            Q_EMIT selectedItemTextPressed(m_pressedIndex);
+        if (m_selectionManager->selectedItems().count() == 1 && m_view->isAboveText(m_pressedIndex.value_or(-1), m_pressedMousePos)) {
+            Q_EMIT selectedItemTextPressed(m_pressedIndex.value_or(-1));
         }
     }
 
@@ -1479,7 +1556,14 @@ bool KItemListController::onPress(const QPoint& screenPos, const QPointF& pos, c
         m_selectionManager->endAnchoredSelection();
     }
 
-    if (buttons & Qt::RightButton) {
+    if (rightClick) {
+
+        // Do header hit check and short circuit before commencing any state changing effects
+        if (m_view->headerBoundaries().contains(pos)) {
+            Q_EMIT headerContextMenuRequested(screenPos);
+            return true;
+        }
+
         // Stop rubber band from persisting after right-clicks
         KItemListRubberBand* rubberBand = m_view->rubberBand();
         if (rubberBand->isActive()) {
@@ -1489,25 +1573,37 @@ bool KItemListController::onPress(const QPoint& screenPos, const QPointF& pos, c
         }
     }
 
-    if (m_pressedIndex >= 0) {
-        m_selectionManager->setCurrentItem(m_pressedIndex);
+    if (m_pressedIndex.has_value()) {
+        m_selectionManager->setCurrentItem(m_pressedIndex.value());
+        const auto row = m_view->m_visibleItems.value(m_pressedIndex.value()); // anything outside of row.contains() will be the empty region of the row rect
+        const bool hitTargetIsRowEmptyRegion = !row->contains(row->mapFromItem(m_view, pos));
+        // again, when this method returns false, a rubberBand selection is created as the event is not consumed;
+        // createRubberBand here tells us whether to return true or false.
+        bool createRubberBand = (hitTargetIsRowEmptyRegion && m_selectionManager->selectedItems().isEmpty());
+
+        if (rightClick && hitTargetIsRowEmptyRegion) {
+            // we got a right click outside the text rect, default to action on the current url and not the pressed item
+            Q_EMIT itemContextMenuRequested(m_pressedIndex.value(), screenPos);
+            return true;
+        }
 
         switch (m_selectionBehavior) {
         case NoSelection:
             break;
 
         case SingleSelection:
-            m_selectionManager->setSelected(m_pressedIndex);
+            m_selectionManager->setSelected(m_pressedIndex.value());
             break;
 
         case MultiSelection:
             if (controlPressed && !shiftPressed) {
-                m_selectionManager->setSelected(m_pressedIndex, 1, KItemListSelectionManager::Toggle);
-                m_selectionManager->beginAnchoredSelection(m_pressedIndex);
+                m_selectionManager->setSelected(m_pressedIndex.value(), 1, KItemListSelectionManager::Toggle);
+                m_selectionManager->beginAnchoredSelection(m_pressedIndex.value());
+                createRubberBand = false; // multi selection, don't propagate any further
             } else if (!shiftPressed || !m_selectionManager->isAnchoredSelectionActive()) {
                 // Select the pressed item and start a new anchored selection
-                m_selectionManager->setSelected(m_pressedIndex, 1, KItemListSelectionManager::Select);
-                m_selectionManager->beginAnchoredSelection(m_pressedIndex);
+                m_selectionManager->setSelected(m_pressedIndex.value(), 1, KItemListSelectionManager::Select);
+                m_selectionManager->beginAnchoredSelection(m_pressedIndex.value());
             }
             break;
 
@@ -1516,20 +1612,16 @@ bool KItemListController::onPress(const QPoint& screenPos, const QPointF& pos, c
             break;
         }
 
-        if (buttons & Qt::RightButton) {
-            Q_EMIT itemContextMenuRequested(m_pressedIndex, screenPos);
+        if (rightClick) {
+            Q_EMIT itemContextMenuRequested(m_pressedIndex.value(), screenPos);
         }
-
-        return true;
+        return !createRubberBand;
     }
 
-    if (buttons & Qt::RightButton) {
-        const QRectF headerBounds = m_view->headerBoundaries();
-        if (headerBounds.contains(pos)) {
-            Q_EMIT headerContextMenuRequested(screenPos);
-        } else {
-            Q_EMIT viewContextMenuRequested(screenPos);
-        }
+    if (rightClick) {
+        // header right click handling would have been done before this so just normal context
+        // menu here is fine
+        Q_EMIT viewContextMenuRequested(screenPos);
         return true;
     }
 
@@ -1538,14 +1630,14 @@ bool KItemListController::onPress(const QPoint& screenPos, const QPointF& pos, c
 
 bool KItemListController::onRelease(const QPointF& pos, const Qt::KeyboardModifiers modifiers, const Qt::MouseButtons buttons, bool touch)
 {
-    const bool isAboveSelectionToggle = m_view->isAboveSelectionToggle(m_pressedIndex, m_pressedMousePos);
+    const bool isAboveSelectionToggle = m_view->isAboveSelectionToggle(m_pressedIndex.value_or(-1), m_pressedMousePos);
     if (isAboveSelectionToggle) {
         m_selectionTogglePressed = false;
         return true;
     }
 
     if (!isAboveSelectionToggle && m_selectionTogglePressed) {
-        m_selectionManager->setSelected(m_pressedIndex, 1, KItemListSelectionManager::Toggle);
+        m_selectionManager->setSelected(m_pressedIndex.value_or(-1), 1, KItemListSelectionManager::Toggle);
         m_selectionTogglePressed = false;
         return true;
     }
@@ -1554,60 +1646,77 @@ bool KItemListController::onRelease(const QPointF& pos, const Qt::KeyboardModifi
     const bool shiftOrControlPressed = modifiers & Qt::ShiftModifier ||
                                        controlPressed;
 
+    const std::optional<int> index = m_view->itemAt(pos);
+
     KItemListRubberBand* rubberBand = m_view->rubberBand();
+    bool rubberBandRelease = false;
     if (rubberBand->isActive()) {
         disconnect(rubberBand, &KItemListRubberBand::endPositionChanged, this, &KItemListController::slotRubberBandChanged);
         rubberBand->setActive(false);
         m_oldSelection.clear();
         m_view->setAutoScroll(false);
+        rubberBandRelease = true;
+        // We check for actual rubber band drag here: if delta between start and end is less than drag threshold,
+        // then we have a single click on one of the rows
+        if ((rubberBand->endPosition() - rubberBand->startPosition()).manhattanLength() < QApplication::startDragDistance()) {
+            rubberBandRelease = false; // since we're only selecting, unmark rubber band release flag
+            // m_pressedIndex will have no value if we came from a multi-selection clearing onPress
+            // in that case, we don't select anything
+            if (index.has_value() && m_pressedIndex.has_value()) {
+                if (controlPressed && m_selectionBehavior == MultiSelection) {
+                    m_selectionManager->setSelected(m_pressedIndex.value(), 1, KItemListSelectionManager::Toggle);
+                } else {
+                    m_selectionManager->setSelected(index.value());
+                }
+                if (!m_selectionManager->isAnchoredSelectionActive()) {
+                    m_selectionManager->beginAnchoredSelection(index.value());
+                }
+            }
+        }
     }
 
-    const int index = m_view->itemAt(pos);
-
-    if (index >= 0 && index == m_pressedIndex) {
+    if (index.has_value() && index == m_pressedIndex) {
         // The release event is done above the same item as the press event
 
         if (m_clearSelectionIfItemsAreNotDragged) {
             // A selected item has been clicked, but no drag operation has been started
             // -> clear the rest of the selection.
             m_selectionManager->clearSelection();
-            m_selectionManager->setSelected(m_pressedIndex, 1, KItemListSelectionManager::Select);
-            m_selectionManager->beginAnchoredSelection(m_pressedIndex);
+            m_selectionManager->setSelected(m_pressedIndex.value(), 1, KItemListSelectionManager::Select);
+            m_selectionManager->beginAnchoredSelection(m_pressedIndex.value());
         }
 
         if (buttons & Qt::LeftButton) {
             bool emitItemActivated = true;
-            if (m_view->isAboveExpansionToggle(index, pos)) {
-                const bool expanded = m_model->isExpanded(index);
-                m_model->setExpanded(index, !expanded);
+            if (m_view->isAboveExpansionToggle(index.value(), pos)) {
+                const bool expanded = m_model->isExpanded(index.value());
+                m_model->setExpanded(index.value(), !expanded);
 
-                Q_EMIT itemExpansionToggleClicked(index);
+                Q_EMIT itemExpansionToggleClicked(index.value());
                 emitItemActivated = false;
-            } else if (shiftOrControlPressed) {
-                // The mouse click should only update the selection, not trigger the item
+            } else if (shiftOrControlPressed && m_selectionBehavior != SingleSelection) {
+                // The mouse click should only update the selection, not trigger the item, except when
+                // we are in single selection mode
                 emitItemActivated = false;
-                // When Ctrl-clicking an item when in single selection mode
-                // i.e. where Ctrl won't change the selection, pretend it was middle clicked
-                if (controlPressed && m_selectionBehavior == SingleSelection) {
-                    Q_EMIT itemMiddleClicked(index);
-                }
-            } else if (!(m_view->style()->styleHint(QStyle::SH_ItemView_ActivateItemOnSingleClick) || m_singleClickActivationEnforced)) {
-                if (touch) {
-                emitItemActivated = true;
+            } else {
+                const bool singleClickActivation = m_view->style()->styleHint(QStyle::SH_ItemView_ActivateItemOnSingleClick) || m_singleClickActivationEnforced;
+                if (!singleClickActivation) {
+                    emitItemActivated = touch;
                 } else {
-                emitItemActivated = false;
+                    // activate on single click only if we didn't come from a rubber band release
+                    emitItemActivated = !rubberBandRelease;
                 }
             }
             if (emitItemActivated) {
-                Q_EMIT itemActivated(index);
+                Q_EMIT itemActivated(index.value());
             }
         } else if (buttons & Qt::MiddleButton) {
-            Q_EMIT itemMiddleClicked(index);
+            Q_EMIT itemMiddleClicked(index.value());
         }
     }
 
     m_pressedMousePos = QPointF();
-    m_pressedIndex = -1;
+    m_pressedIndex = std::nullopt;
     m_clearSelectionIfItemsAreNotDragged = false;
     return false;
 }
