@@ -8,12 +8,14 @@
 
 #include <QTest>
 #include <QSignalSpy>
+#include <QStandardPaths>
 
 class KItemListKeyboardSearchManagerTest : public QObject
 {
     Q_OBJECT
 
-private slots:
+private Q_SLOTS:
+    void initTestCase();
     void init();
 
     void testBasicKeyboardSearch();
@@ -25,6 +27,11 @@ private:
     KItemListKeyboardSearchManager m_keyboardSearchManager;
 };
 
+void KItemListKeyboardSearchManagerTest::initTestCase()
+{
+    QStandardPaths::setTestModeEnabled(true);
+}
+
 void KItemListKeyboardSearchManagerTest::init()
 {
     // Make sure that the previous search string is cleared
